from saml_service_provider.settings import SAMLServiceProviderSettings


class LastPassServiceProviderSettings(SAMLServiceProviderSettings):
    def __init__(self, onelogin_connector_id, onelogin_x509_cert=None,
                 onelogin_x509_fingerprint=None, **kwargs):
        self.contact_info = {
            "technical": {
                "givenName": "Admin",
                "emailAddress": kwargs.get('SAML_SP_ADMIN_EMAIL',
                                           'admin@example.com')
            }
        }

        if 'contact_info' in kwargs:
            self.contact_info = kwargs.pop('contact_info')

        if 'organization_info' in kwargs:
            self.organization_info = kwargs.pop('organization_info')

        conf = dict()
        conf['idp_metadata_url'] = 'https://lastpass.com/saml/login/{}'.format(
            onelogin_connector_id
        )
        conf['idp_sso_url'] = 'https://lastpass.com/saml/login/{}'.format(
            onelogin_connector_id
        )
        conf['idp_slo_url'] = 'https://lastpass.com/saml/logout/{}'.format(
            onelogin_connector_id
        )
        if onelogin_x509_cert:
            conf['idp_x509cert'] = onelogin_x509_cert
        elif onelogin_x509_fingerprint:
            conf['idp_x509_fingerprint'] = onelogin_x509_fingerprint
        else:
            raise Exception('Provide either cert or fingerprint')

        kwargs.update(conf)
        super(LastPassServiceProviderSettings, self).__init__(**kwargs)
        self.settings['sp']['NameIDFormat'] = "urn:oasis:names:tc:SAML:2.0:name-id-format:emailAddress"



from django.conf.urls import url
from django.views.decorators.csrf import csrf_exempt
from lpsp.views import (MetadataView, CompleteAuthenticationView,
                      InitiateAuthenticationView)

urlpatterns = [
    url(r'^initiate-login/$', InitiateAuthenticationView.as_view(),
        name="saml_login_initiate"),
    url(r'^complete-login/$', csrf_exempt(CompleteAuthenticationView.as_view()),
        name="saml_login_complete"),
    url(r'^metadata/$', MetadataView.as_view(), name='saml_metadata'),
]



from django.conf import settings
from django.contrib.auth.models import User


class SAMLServiceProviderBackend(object):

    def authenticate(self, saml_authentication=None):
        if not saml_authentication:  # Using another authentication method
            return None

        if saml_authentication.is_authenticated():
            attributes = saml_authentication.get_attributes()
            print dir(saml_authentication)
            print attributes
            try:
                email = saml_authentication.get_nameid()
                user = User.objects.get(username=email.split('@')[0])
            except User.DoesNotExist:
                superuser = getattr(settings, 'LASTPASS_CREATE_ADMIN', False)
                staff = getattr(settings, 'LASTPASS_CREATE_STAFF', False)
                user = User(username=email.split('@')[0], email=email,
                            is_superuser=superuser, is_staff=staff)
                user.set_unusable_password()
                user.save()
            return user
        return None

    def get_user(self, user_id):
        try:
            return User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return None



from django.conf import settings
from django.core.urlresolvers import reverse

from saml_service_provider import views as samlviews
from .settings import LastPassServiceProviderSettings


class SettingsMixin(object):
    def get_onelogin_settings(self):
        conf = dict()
        conf['onelogin_connector_id'] = settings.LASTPASS_CONNECTOR_ID
        if hasattr(settings, 'LASTPASS_CERTIFICATE'):
            conf['onelogin_x509_cert'] = settings.LASTPASS_CERTIFICATE
        elif hasattr(settings, 'LASTPASS_CERTIFICATE_FILE'):
            with open(settings.LASTPASS_CERTIFICATE_FILE, 'rb') as cert:
                conf['onelogin_x509_cert'] = cert.read()
        else:
            conf['onelogin_x509_fingerprint'] = settings.LASTPASS_FINGERPRINT

        conf['sp_metadata_url'] = 'http://localhost:8000{}'.format(
            reverse('saml_metadata')
        )
        conf['sp_login_url'] = 'http://localhost:8000{}'.format(
            reverse('saml_login_complete')
        )
        conf['sp_logout_url'] = 'http://localhost:8000{}'.format(
            reverse('logout')
        )
        conf['debug'] = settings.DEBUG
        conf['strict'] = not settings.DEBUG

        if hasattr(settings, 'SAML_SP_CERTIFICATE'):
            conf['sp_x509cert'] = settings.SAML_SP_CERTIFICATE
        else:
            with open(settings.SAML_SP_CERTIFICATE_FILE, 'rb') as cert:
                conf['sp_x509cert'] = cert.read()

        if hasattr(settings, 'SAML_SP_KEY'):
            conf['sp_private_key'] = settings.SAML_SP_KEY
        else:
            with open(settings.SAML_SP_KEY_FILE, 'rb') as key:
                conf['sp_private_key'] = key.read()

        if hasattr(settings, 'SAML_SP_CONTACT_INFO'):
            conf['contact_info'] = settings.SAML_SP_CONTACT_INFO

        if hasattr(settings, 'SAML_SP_ORGANIZATION_INFO'):
            conf['organization_info'] = settings.SAML_SP_ORGANIZATION_INFO

        return LastPassServiceProviderSettings(**conf).settings


class InitiateAuthenticationView(SettingsMixin,
                                 samlviews.InitiateAuthenticationView):
    pass


class CompleteAuthenticationView(SettingsMixin,
                                 samlviews.CompleteAuthenticationView):
    pass


class MetadataView(SettingsMixin,
                   samlviews.MetadataView):
    pass



#!/usr/bin/env python

import os
import versioneer
from setuptools import setup
from pip.req import parse_requirements
from pip.download import PipSession


def get_requirements(filename):
    if not os.path.exists(filename):
        return []

    install_reqs = parse_requirements(filename, session=PipSession())
    return [str(ir.req) for ir in install_reqs]

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

setup(name='django-lastpass-sp',
      version=versioneer.get_version(),
      cmdclass=versioneer.get_cmdclass(),
      description='Django LastPass SAML authenticator',
      long_description=README,
      author='Ferrix Hovi',
      author_email='ferrix@codetry.fi',
      install_requires=get_requirements('requirements.txt'),
      tests_require=get_requirements('development.txt'),
      packages=['lpsp'],
      url='https://github.com/codetry/lpsp/',
      license='MIT License',
      classifiers=[
          'Environment :: Web Environment',
          'Framework :: Django',
          'Framework :: Django :: 1.8',
          'Intended Audience :: Developers',
          'Operating System :: OS Independent',
          'Programming Language :: Python',
          'License :: OSI Approved :: MIT License',
          'Topic :: Internet :: WWW/HTTP',
          'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
      ],
      )
